//
//  main.c
//
//  Program main / SANWA PC20 SDcard logger 0.3
//  Copyright (C) 2010 Toyohiko Togashi tog001@nifty.com
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		DMM(Digital Multi Meter) SANWA PC20 measuring data logger.
//		Record to SD card memory.
//
//	Setting
//		AVR
//			Device	: ATMEL AVR ATmega328P
//			IDE		: AVR Studio 4.18.700  
//			Compiler: WinAVR 20100110(GCC 4.3.3)
//	Usage
//		Insert SDcard memory. (FAT16 format)
//
//		SDcard output file:
//			File name	: "/PC20.CSV"
//			Format		: CSV(camma separated value)
//				"TimeStamp, AC/DC, Value, Unit"
//		Parameter file: <Optional>
//			File name	: "/PC20.TXT"
//			Format		:
//			+------------------------------
//			|yy=<yyyy>	[#<comment>]
//			|mm=<mm>	[#<comment>]
//			|dd=<dd>	[#<comment>]
//			|hh=<hh>	[#<comment>]
//			|mi=<mi>	[#<comment>]
//			|ss=<ss>	[#<comment>]
//			+------------------------------
//				<yyyy>	: Set now date, years
//				<mm>	: Set now date, month#
//				<dd>	: Set now date, day of the month
//				<hh>	: Set now time, hours
//				<mi>	: Set now time, minits
//				<ss>	: Set now time, seconds
//				<comment>: Option text
//
//
//	Update history
//	---------- ----- -------------------------------------------
//	2010.07.18 v0.0  First cording
//	2010.08.22 v0.3  First release (base version)
//
//

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "avr/io.h"
#include "avr/interrupt.h"
#include "avr/sleep.h"

#include "sd.h"


//---------------------------------------------------------------------
//	Constant area
//---------------------------------------------------------------------
#define F_CPU 	1000000UL  // 1 MHz
#define SD_CD()		!(PINB & _BV(PINB1))
#define LED_1_ON()	PORTD &= ~_BV(PORTD2)
#define LED_1_OFF()	PORTD |=  _BV(PORTD2)
#define LED_2_ON()	PORTD &= ~_BV(PORTD3)
#define LED_2_OFF()	PORTD |=  _BV(PORTD3)
#define LED_3_ON()	PORTD &= ~_BV(PORTD4)
#define LED_3_OFF()	PORTD |=  _BV(PORTD4)
#define SWITCH_1()	!(PIND & _BV(PIND5))


//---------------------------------------------------------------------
//	Global area
//---------------------------------------------------------------------
unsigned char	tout = 0;				// Recieve timeout
char 			buf[80];				// Share


//==============================================================================
//
//	Interrupt Service Routine
//
//==============================================================================

//
//	Real time clock
//
// 1 sec interval timer ISR
unsigned char 	ddom[] = {0, 31,28,31,30,31,30,31,31,30,31,30,31};
short 			yy = 2010;
unsigned char 	mm = 8;
unsigned char 	dd = 1;
unsigned char 	hh = 0;
unsigned char 	mi = 0;
unsigned char 	ss = 0;

ISR(TIMER2_OVF_vect) {
//	if (TIFR2 & _BV(TOV2)) {

		LED_1_ON();

		TIFR2 &= ~_BV(TOV2);

		ss++;
		if (ss > 59) {
			ss = 0;
			mi++;
			if (mi > 59) {
				mi = 0;
				hh++;
				if (hh > 23) {
					hh = 0;
					dd++;
					if (((yy & 0x03) == 0) && ((yy % 100) != 0)) {
						ddom[2] = 29;
					} else {
						ddom[2] = 28;
					}
					if (dd > ddom[mm]) {
						dd = 1;
						mm++;
						if (mm > 12) {
							mm = 1;
							yy++;
						}
					}
				}
			}
		}

		// Count timeout
		if (tout < 200) {
			tout++;
		}

		LED_1_OFF();
//	}
	return;
}

//
// DMM data 1byte Recieve
//
#define RCVSIZE 32
#define WRTSIZE 16
static struct {							// DMM data
	unsigned char 	dat[16];			// raw data
	short 			yy;					// time stamp
	char 			mm;					// time stamp
	char 			dd;					// time stamp
	char 			hh;					// time stamp
	char 			mi;					// time stamp
	char 			ss;					// time stamp
	unsigned char	tm;					// sec/128
} rcv[RCVSIZE];
static int 	rp = 0;						// recieve point
static int 	cp = 0;						// check point

ISR(USART_RX_vect) {

	LED_2_ON();
	if (UCSR0A & _BV(RXC0)) {
		unsigned char	c;
		unsigned char	i;

		c = UDR0;
		UCSR0A &= ~_BV(RXC0);

		i = c >> 4;
		rcv[rp].dat[i] = c & 0xf;

		// Packet termination
		if (i == 0xe) {

			// Time stamp
			rcv[rp].yy = yy;
			rcv[rp].mm = mm;
			rcv[rp].dd = dd;
			rcv[rp].hh = hh;
			rcv[rp].mi = mi;
			rcv[rp].ss = ss;
			rcv[rp].tm = TCNT2;			// Timer2

			// Next entry
			rp++;
			if (rp >= RCVSIZE) {
				rp = 0;
			}

		// Time sync packet
		} else if (i == 0xf) {
			if (rcv[rp].dat[2] == 0x05) {	// Check ID
				yy    =(rcv[rp].dat[ 0] << 4 | rcv[rp].dat[ 1]) + 2000;
				mm    = rcv[rp].dat[ 3];
				dd    = rcv[rp].dat[ 4] << 4 | rcv[rp].dat[ 5];
				hh    = rcv[rp].dat[ 6] << 4 | rcv[rp].dat[ 7];
				mi    = rcv[rp].dat[ 8] << 4 | rcv[rp].dat[ 9];
				ss    = rcv[rp].dat[10] << 4 | rcv[rp].dat[11];
				TCNT2 = rcv[rp].dat[12] << 4 | rcv[rp].dat[13];
				LED_3_ON();
			}
		}

		// Reset timeout
		tout = 0;

	}
	LED_2_OFF();
	return;
}

//==============================================================================
//
//	Subroutines
//
//==============================================================================

//
// 7segment to char data
//
unsigned char segconv(int a, int b) {
	// 7seg patturn
	//
	//  aaa
	// f   b
	// f   b      76543210
	//  ggg   ==> 0efadcgb
	// e   c
	// e   c
	//  ddd
	//
	const int pat[] = { 0x7d, 0x05, 0x5b, 0x1f, 0x27, 0x3e, 0x7e, 0x15, 0x7f, 0x3f, 0x68, 0x6e, 0x35, 0x37, 0x00, 0x80 };
	const int val[] = {  '0',  '1',  '2',  '3',  '4',  '5',  '6',  '7',  '8',  '9',  'L',  '6',  '7',  '9',  ' ',  '*' };
	int i;
	int c;

	c = ((a & 0x7) << 4) | (b & 0x0f);
	for(i = 0; pat[i] < 0x80; i++) {
		if (pat[i] == c) break;
	} 
	return val[i];
}

//
// convert to char data
//
void valconv(unsigned char *buff, char *a, char *s, char *u) {
	char n[5];		// digit

	// AC/DC
	a[0] = '\0';
	if (buff[1] & 0x08) {
		strcat(a, "AC");
	} else if (buff[1] & 0x04) {
		strcat(a, "DC");
	}

	// 4digit
	n[0] = segconv(buff[2], buff[3]);
	n[1] = segconv(buff[4], buff[5]);
	n[2] = segconv(buff[6], buff[7]);
	n[3] = segconv(buff[8], buff[9]);
	n[4] = '\0';

	// N/A data
	if (n[0] == ' ') {
		strcpy(s, "N/A");

	// normal data
	} else {
		int f;
		int i;
		int j;
		
		// float point
		if (buff[4] & 0x08) {
			f = 0;
		} else if (buff[6] & 0x08) {
			f = 1;
		} else if (buff[8] & 0x08) {
			f = 2;
		} else {
			f = 3;
		}
			
		// zero supprelss
		for(i = 0; i < f; i++) {
			if (n[i] != '0') break;
		}
			
		// add sign
		j = 0;
		if (buff[2] & 0x08) {
			s[j++] = '-';
		}
			
		// insert dot
		for(; i <= 5; i++) {
			s[j++] = n[i];
			if (i == f) {
				if (f < 3) {
					s[j++] = '.';
				}
			}
		}
	}
			
	// unit
	u[0] = '\0';
	if (buff[11] & 0x02) {
		strcat(u, "M");
	} else if (buff[10] & 0x02) {
		strcat(u, "k");
	} else if (buff[11] & 0x08) {
		strcat(u, "m");
	} else if (buff[10] & 0x08) {
		strcat(u, "u");
	} else if (buff[10] & 0x04) {
		strcat(u, "n");
	}

	if (buff[12] & 0x08) {
		strcat(u, "F");
	} else if (buff[12] & 0x04) {
		strcat(u, "Ohm");
	} else if (buff[13] & 0x08) {
		strcat(u, "A");
	} else if (buff[13] & 0x04) {
		strcat(u, "V");
	} else if (buff[13] & 0x02) {
		strcat(u, "Hz");
	}

	return;
}

//
// SD card write
//
void burstwrite(int fd) {
	int i; 
	int	j;

	LED_3_ON();

	j = rp;
	i = cp;
	while(i != j) {
		char a[4];		// AC/DC
		char s[16];		// edited number
		char u[8];		// unit

		// time stamp

		// values
		valconv(rcv[i].dat, a, s, u);

		// write
		sprintf(buf, "\"%d-%02d-%02d %02d:%02d:%02d.%1d\", %s, %s, %s\r\n",
			rcv[i].yy, rcv[i].mm, rcv[i].dd,
			rcv[i].hh, rcv[i].mi, rcv[i].ss,
			rcv[i].tm/26,
			a, s, u);
		sd_write(fd, buf, strlen(buf));
		
		// next
		i++;
		if (i >= RCVSIZE) {
			i = 0;
		}

	}
	sd_sync(fd);
	cp = j;

	LED_3_OFF();

}

//
// SD card read
//
unsigned char sd_gets(int fd, char *buff, unsigned char sz){
	char	c[1];
	int		n;

	while((n = sd_read(fd, c, sizeof(c))) > 0) {
		if (sz > 1) {
			--sz;
			*buff++ = c[0];
		}
		if (c[0] == '\n') {
			break;
		}
	}
	*buff = '\0';
	return(n);
}


//==============================================================================
//
//	Main
//
//==============================================================================
int main(void) {
	int		fd;
	char 	csv_fname[] = "PC20.CSV";
	char 	cfg_fname[] = "PC20.TXT";

	//
	// CPU
	//
	set_sleep_mode(SLEEP_MODE_IDLE);	// Peripheral is active

	//
	// I/O port
	//
	LED_1_ON();
	LED_2_ON();
	LED_3_ON();
	DDRD  |=  _BV(DDD2);				// Status R-LED, output
	DDRD  |=  _BV(DDD3);				// Status G-LED, output
	DDRD  |=  _BV(DDD4);				// Status B-LED, output

	DDRD  &= ~_BV(DDD5);				// Push switch, input
	PORTD |=  _BV(PORTD5);				//   enable internal pull-ups

	DDRB  &= ~_BV(DDB1);				// SDcard detect, input
	PORTB &= ~_BV(PORTB1);				//   disable internal pull-ups
	PORTB |=  _BV(PORTB4);				//   SPI-MISO enable pull-ups

	PORTD |=  _BV(PORTD0);				// USART0-RXD enable internal pull-ups

	//
	// USART 0 device
	//
	DDRD  &= ~_BV(DDD0);				// RXD input
	DDRD  |=  _BV(DDD1);				// TXD output
	UCSR0A =							// USART Control and Status Register 0 A
//			_BV(RXC0)  |				// USART Receive Complete
//			_BV(TXC0)  |				// USART Transmit Complete
//			_BV(UDRE0) |				// USART Data Register Empty
//			_BV(FE0)   |				// Frame Error
//			_BV(DOR0)  |				// Data OverRun
//			_BV(UPE0)  |				// USART Parity Error
//			_BV(U2X0)  |				// Double the USART Transmission Speed
//			_BV(MPCM0) |				// Multi-processor Communication Mode
			0;							//
	UCSR0B =							// USART Control and Status Register 0 B
			_BV(RXCIE0) |				// RX Complete Interrupt Enable
//			_BV(TXCIE0) |				// TX Complete Interrupt Enable
//			_BV(UDRIE0) |				// USART Data Register Empty Interrupt Enable
			_BV(RXEN0)  |				// Receiver Enable
			_BV(TXEN0)  |				// Transmitter Enable
//			_BV(UCSZ02) |				// Character Size => UCSR0C
//			_BV(RXB80)  |				// Receive Data Bit 8
//			_BV(TXB80)  |				// Transmit Data Bit 8
			0;							//
	UCSR0C =							// USART Control and Status Register 0 C
//			_BV(UMSEL01) |				// USART Mode Select  0  0  1  1
//			_BV(UMSEL00) |				//                    0  1  0  1
										//                    +--|--|--|-- Asynchronous USART  <==
										//                       +--|--|-- Synchronous USART
										//                          +--|-- Reserved
										//                             +-- Master SPI (MSPIM)
//			_BV(UPM01)   |				// Parity Mode        0  0  1  1
//			_BV(UPM00)   |				//                    0  1  0  1
										//                    +--|--|--|-- Disabled  <==
										//                       +--|--|-- Reserved
										//                          +--|-- Enabled, Even Parity
										//                             +-- Enabled, Odd Parity
//			_BV(USBS0)   |				// Stop Bit Select    0:1-bit  1:2-bit
//			_BV(UCSZ02) <= UCSR0B		// Character Size     0  0  0  0  1  1  1  1
			_BV(UCSZ01)  |				//                    0  0  1  1  0  0  1  1
			_BV(UCSZ00)  |				//                    0  1  0  1  0  1  0  1
										//                    +--|--|--|--|--|--|--|-- 5-bit
										//                       +--|--|--|--|--|--|-- 6-bit
										//                          +--|--|--|--|--|-- 7-bit
										//                             +--|--|--|--|-- 8-bit    <==
										//                                +--|--|--|-- Reserved
										//                                   +--|--|-- Reserved
										//                                      +--|-- Reserved
										//                                         +-- 9-bit
//			_BV(UCPOL0)  |				// Clock Polarity     0:Tx=Rising,Rx=Falling 1:Tx=Falling,Rx=Rising Edge
			0;							//
	UBRR0 = F_CPU / 16 / 2400 - 1;		// Baud rate <== 2400bps

	//
	// Timer2
	//
//	DDRB  |=  _BV(DDB3);				// OC2A port output
//	DDRD  |=  _BV(DDD3);				// OC2B port output
	TCCR2A =							// Timer/Counter Control Register A
										// Compare Match Output A Mode
//			_BV(COM2A1) |				//   Compare Output, non-PWM Mode            0  0  1  1
//			_BV(COM2A0) |				//                                           0  1  0  1
										//                                           +--|--|--|-- Normal port operation, OC0A disconnected.  <==
										//                                              +--|--|-- Toggle OC2A on Compare Match
										//                                                 +--|-- Clear OC2A on Compare Match
										//                                                    +-- Set OC2A on Compare Match
//			_BV(COM2A1) |				//   Compare Output, Fast PWM Mode           0  0  1  1
//			_BV(COM2A0) |				//                                           0  1  0  1
										//                                           +--|--|--|-- Normal port operation, OC2A disconnected.
										//                                              +--|--|-- WGM22=0: Normal Port Operation, OC0A Disconnected. WGM22=1: Toggle OC2A on Compare Match.
										//                                                 +--|-- Clear OC2A on Compare Match, set OC2A at BOTTOM,(non-inverting mode).
										//                                                    +-- Set OC2A on Compare Match, clear OC2A at BOTTOM,(invertig mode).
//			_BV(COM2A1) |				//   Compare Output, Phase Correct PWM Mode  0  0  1  1
//			_BV(COM2A0) |				//                                           0  1  0  1
										//                                           +--|--|--|-- Normal port operation, OC2A disconnected.
										//                                              +--|--|-- WGM22=0: Normal Port Operation, OC2A Disconnected. WGM22=1: Toggle OC2A on Compare Match.
										//                                                 +--|-- Clear OC2A on Compare Match when up-counting. Set OC2A on Compare Match when down-counting.
										//                                                    +-- Set OC2A on Compare Match when up-counting. Clear OC2A on Compare Match when down-counting.
										// Compare Match Output B Mode
//			_BV(COM2B1) |				//   Compare Output, non-PWM Mode            0  0  1  1
//			_BV(COM2B0) |				//                                           0  1  0  1
										//                                           +--|--|--|-- Normal port operation, OC2B disconnected.  <==
										//                                              +--|--|-- Toggle OC2B on Compare Match
										//                                                 +--|-- Clear OC2B on Compare Match
										//                                                    +-- Set OC2B on Compare Match
//			_BV(COM2B1) |				//   Compare Output, Fast PWM Mode           0  0  1  1
//			_BV(COM2B0) |				//                                           0  1  0  1
										//                                           +--|--|--|-- Normal port operation, OC2B disconnected.
										//                                              +--|--|-- Reserved
										//                                                 +--|-- Clear OC2B on Compare Match, set OC2B at BOTTOM, (non-inverting mode).
										//                                                    +-- Set OC2B on Compare Match, clear OC2B at BOTTOM, (inverting mode).
//			_BV(COM2B1) |				//   Compare Output, Phase Correct PWM Mode  0  0  1  1
//			_BV(COM2B0) |				//                                           0  1  0  1
										//                                           +--|--|--|-- Normal port operation, OC2B disconnected.
										//                                              +--|--|-- Reserved
										//                                                 +--|-- Clear OC2B on Compare Match when up-counting. Set OC2B on Compare Match when down-counting.
										//                                                    +-- Set OC2B on Compare Match when up-counting. Clear OC2B on Compare Match when down-counting.
//			_BV(WGM22) <= TCCR2B		// Waveform Generation Mode  0  0  0  0  1  1  1  1
//			_BV(WGM21)  |				//                           0  0  1  1  0  0  1  1
//			_BV(WGM20)  |				//                           0  1  0  1  0  1  0  1
										//                           +--|--|--|--|--|--|--|-- Mode0, Normal  <==
										//                              +--|--|--|--|--|--|-- Mode1, PWM, Phase Correct
										//                                 +--|--|--|--|--|-- Mode2, CTC
										//                                    +--|--|--|--|-- Mode3, Fast PWM
										//                                       +--|--|--|-- Mode4, Reserved
										//                                          +--|--|-- Mode5, PWM, Phase Correct
										//                                             +--|-- Mode6, Reserved
										//                                                +-- Mode7, Fast PWM
			0;							//
	TCCR2B =							// Timer/Counter Control Register B
//			_BV(FOC2A) |				// Force Output Compare A
//			_BV(FOC2B) |				// Force Output Compare B
//			_BV(WGM22) |				// Waveform Generation Mode => See TCCR2A
			_BV(CS22)  |				// Clock Select              0  0  0  0  1  1  1  1
//			_BV(CS21)  |				//                           0  0  1  1  0  0  1  1
			_BV(CS20)  |				//                           0  1  0  1  0  1  0  1
										//                           +--|--|--|--|--|--|--|-- No clock source (Timer/Counter stopped).
										//                              +--|--|--|--|--|--|-- clk T2S
										//                                 +--|--|--|--|--|-- clk T2S/8
										//                                    +--|--|--|--|-- clk T2S/32
										//                                       +--|--|--|-- clk T2S/64
										//                                          +--|--|-- clk T2S/128  <==
										//                                             +--|-- clk T2S/256
										//                                                +-- clk T2S/1024
			0;							//
	TCNT2 =	0;							// Timer/Counter Register
	OCR2A =	0;							// Output Compare Register A
	OCR2B =	0;							// Output Compare Register B
	TIMSK2 =							// Timer/Counter2 Interrupt Mask Register
//			_BV(OCIE2B) |				// Output Compare Match B Interrupt Enable
//			_BV(OCIE2A) |				// Output Compare Match A Interrupt Enable
			_BV(TOIE2)  |				// Overflow Interrupt Enable
			0;							//
	TIFR2 =								// Timer/Counter2 Interrupt Flag Register
//			_BV(OCF2B)  |				// Output Compare Flag 2 B
//			_BV(OCF2A)  |				// Output Compare Flag 2 A
//			_BV(TOV2)   |				// Overflow Flag
			0;							//
	ASSR =								// Asynchronous Status Register
//			_BV(EXCLK)   |				// Enable External Clock Input
			_BV(AS2)     |				// Asynchronous Timer/Counter2
//			_BV(TCN2UB)  |				// Timer/Counter2 Update Busy
//			_BV(OCR2AUB) |				// Output Compare Register2 Update Busy
//			_BV(OCR2BUB) |				// Output Compare Register2 Update Busy
//			_BV(TCR2AUB) |				// Timer/Counter Control Register2 Update Busy
//			_BV(TCR2BUB) |				// Timer/Counter Control Register2 Update Busy
			0;							//
	GTCCR |= _BV(PSRASY);				// Prescaler Reset Timer/Counter2 (General Timer/Counter Control Register)

	LED_1_OFF();
	LED_2_OFF();
	LED_3_OFF();


	//
	// Main loop
	//
	fd = -1;
	sei();

	for(;;){

		//
		// Check SDcard
		//

		// Card Detect
		if (SD_CD()) {

			// Insert
			if (fd < 0) {

				// Mount
				LED_1_OFF();
				if (sd_mount(0) < 0) {
					LED_1_ON();
				} else {

					// Read parameters
					fd = sd_open(cfg_fname, 0, 0);
					if (fd >= 0) {
						while(sd_gets(fd, buf, sizeof(buf))) {
							static char 	*prm_label[] =
								{"yy", "mm", "dd", "hh", "mi", "ss", NULL};
							char			*p;
							int				v;
							unsigned char	i;

							p = strtok(buf, "= \t\n");			// Label
							v = atoi(strtok(NULL, ", \t\n#"));	// Value
							for(i = 0; prm_label[i] != NULL; i++) {
								if (!strcmp(prm_label[i], p)) {
									break;
								}
							}
							if (prm_label[i] != NULL) {
								switch(i) {
								case 0:		yy	= v;	break;
								case 1:		mm	= v;	break;
								case 2:		dd	= v;	break;
								case 3:		hh	= v;	break;
								case 4:		mi	= v;	break;
								case 5:		ss	= v;	break;
								default:				break;
								}
							}
						}
						sd_close(fd);
					}

					// Open CSV file
					SD_SET_DATE(yy, mm, dd);
					SD_SET_TIME(hh, mi, ss);
					if ((fd = sd_open(csv_fname, SD_O_CREAT|SD_O_APPEND, 0)) < 0) {
						LED_1_ON();
					} else {
						sprintf(buf, "TimeStamp, AC/DC, Value, Unit\r\n");
						sd_write(fd, buf, strlen(buf));
					}
				}

				// Power active
				set_sleep_mode(SLEEP_MODE_IDLE);
			}

		// Not detect
		} else {

			// Eject
			if (fd >= 0) {

				// Close
				sd_close(fd);

				// Unmount
				sd_unmount();
				fd = -1;

				// Power save
				set_sleep_mode(SLEEP_MODE_PWR_SAVE);
			}

		}
			
		//
		// Write SDcard
		//
		if(fd >= 0) {
			int d;

			// Check data full
			d = rp - cp;
			if (d < 0) {
				d += RCVSIZE;
			}
			if (d >= WRTSIZE) {
				burstwrite(fd);
			}

			// DMM poweroff detect
			if (tout > 3) {

				// Flush data
				burstwrite(fd);

				// Power save
				set_sleep_mode(SLEEP_MODE_PWR_SAVE);

			} else {

				// Power active
				set_sleep_mode(SLEEP_MODE_IDLE);

			}

		}

		//
		// Send time sync packet
		//
		if (SWITCH_1()) {
			unsigned char	y;
			unsigned char	i;
			char	*p;

			y = yy - 2000;
			buf[ 0] = 0x00 | (y >> 4);
			buf[ 1] = 0x10 | (y & 0xf);
			buf[ 2] = 0x25;
			buf[ 3] = 0x30 | (mm & 0xf);
			buf[ 4] = 0x40 | (dd >> 4);
			buf[ 5] = 0x50 | (dd & 0xf);
			buf[ 6] = 0x60 | (hh >> 4);
			buf[ 7] = 0x70 | (hh & 0xf);
			buf[ 8] = 0x80 | (mi >> 4);
			buf[ 9] = 0x90 | (mi & 0xf);
			buf[10] = 0xa0 | (ss >> 4);
			buf[11] = 0xb0 | (ss & 0xf);
			buf[12] = 0xc0 | (TCNT2 >> 4);
			buf[13] = 0xd0 | (TCNT2 & 0xf);
			buf[14] = 0xf0;

			p = buf;
			for(i = 15; i > 0; --i) {
				UDR0 = *p++;
				while (!(UCSR0A & _BV(UDRE0)));
			}
			while (!(UCSR0A & _BV(TXC0)));
			for(i = 200; i > 0; --i){	// Wait for transmitt
				LED_2_ON();
				LED_3_ON();
				LED_2_OFF();
				LED_3_OFF();
			}
		}

		//
		// Wait event
		//
		sleep_mode();

	}

}
